﻿#include "animation.h"
#include <fstream>
#include <iostream>
#include <sstream>

using namespace std;

Animation::Animation()
{
}

void Animation::openFile(const std::string& filepath)
{
    std::ifstream file;
    file.open(filepath); //open the input file
    if (!file.fail())
    {
        frames.clear();
        frameId = 0;

        std::stringstream strStream;
        strStream << file.rdbuf(); //read the file
        std::string filestr = strStream.str(); //str holds the content of the file
        file.close();

        size_t start = 0;
        while (1) {
            size_t begin = filestr.find("frame(", start);
            size_t end = filestr.find(")", begin) + 1;
            if (begin == string::npos || end == string::npos)
                break;
            start = end;
            frames.push_back(MxFrame(filestr.substr(begin, end - begin)));
        }
    }
}

bool Animation::eof() const
{
    return frameId >= frames.size();
}

MxFrame Animation::nextFrame()
{
    if (!eof()) {
        return frames[frameId++];
    } else {
        return MxFrame();
    }
}

MxFrame Animation::getCurrentFrame() const
{
    if (!eof()) {
        return frames[frameId];
    } else {
        return MxFrame();
    }
}

uint32_t Animation::getCurrentTime() const
{
    uint32_t t = 0;
    if (frameId == 0) return 0;
    for (uint32_t i = 0; i < frameId - 1; i++) {
        t += frames[i].length;
    }
    return t;
}

uint32_t Animation::getTimeLength() const
{
    uint32_t t = 0;
    for (uint32_t i = 0; i < frames.size(); i++) {
        t += frames[i].length;
    }
    return t;
}

uint32_t Animation::getFrameNum() const
{
    return static_cast<uint32_t>(frames.size());
}

uint32_t Animation::getFrameId() const
{
    return frameId;
}

uint32_t Animation::setFrameId(uint32_t id)
{
    frameId = id < frames.size() ? id : frames.size();
    return frameId;
}

//returns: (time_param) - (actual beginning time of set frame)
uint32_t Animation::seek(uint32_t time_ms)
{
    uint32_t t = 0;
    uint32_t i = 0;
    for (i = 0; i < frames.size(); i++) {
        if (t + frames[i].length > time_ms)
            break;
        t += frames[i].length;
    }
    frameId = i;
    return time_ms - t;
}
